/******************************************************************************
* CJDD.G
* computes numerical approximations to the data density based on MCMC output
* by Chib and Jeliazkov (2001)
*
* upadted:  06/23/2005
******************************************************************************/
new;
/** set seed **/
/** rndseed 1234; **/


/********************************************************
* House cleaning and set defaults
********************************************************/
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
_verbose = 1;       /** 1 for verbose **/

pmsel   = "";

#include nkmp_mod.src;
#include nkmp_para.src;
#include nkmp_spec.g;

#include pathspec.g;

/** start log file **/
outstr = resupath $+ "\\cjdd.log";
output file = ^outstr reset;

#include head.g;


/********************************************************
* Configure the Metropolis algorithm
********************************************************/
mhrun   = "12";
cc      = 0.3;  
/* 
** use cc = 0.15 for usm11t0p1
*/


/** initialize output file **/
opath   = parapath $+ "\\mhrun" $+ mhrun;
chkdir  = chk_dir(opath);
ocjdd   = opath $+ "\\cjdd";
ocjdda1   = opath $+ "\\cjdda1";
ocjdda2q2 = opath $+ "\\cjdda2q2";

create fhcjdd=^ocjdd with cjdd, 1, 8;
create fhcjdda1=^ocjdda1 with cjdda1, 1, 8;
create fhcjdda2q2=^ocjdda2q2 with cjdda2q2, 1, 8;

/******************************************************************
* Step 1: 1/J * \sum_{j=1}^J a(para(*),para(j)|Y)
******************************************************************/

/** set environment **/
jsim = 100*10000;

/** initialize **/
a1 = zeros(jsim,1);

/** print process **/
if _verbose==1;
    "";
    "STEP 1: ";
    "";
endif;

j=1;
do until j > jsim;

    /** draw from proposal density **/
    paraj = para + cc*(sigscale*rndn(npara,1));
    paraj = paraj.*pmaskinv + pfix.*pmask;

    /** evaluate at para(j) **/
    {postj, likej} = fcn2(paraj);

    /**  a(para*,para(j)|Y)  **/
    a1[j] = minc(1 | exp( postj - post[1]));


   /** print process **/
   if _verbose==1;
        if fmod(j,100)==0; ".";; pause(1e-20); endif;
        if fmod(j,5000)==0; ""; j; pause(1e-20); endif;
    endif;
    
    j = j+1;

endo;

writer(fhcjdda1,a1);


/******************************************************************
* Step 2: 1/M \sum_{g=1}^M a(para(g),para(*)|Y)*q(para(g),para(*)|Y)
******************************************************************/

/** set environments **/
nburn   = 1;    /** initial draws for burning**/
nblock  = 10000;    /** read block size **/


/** print process **/
if _verbose==1;
    "";
    "STEP 2: ";
    "";
endif;


/************************************************
* Load Parameter Draws from MH Output
************************************************/

lpath = parapath $+ "\\mhrun" $+ mhrun;
lstat = lpath $+ "\\st";
lpara = lpath $+ "\\pa";

/** check Hessian results **/
if not(exist(lpara $+ ".dat"));
    "";
    "Please run Metropolis-Hastings algorithm first.";
    "";
    end;
endif;

open fhstat = ^lstat for read;
open fhpara = ^lpara for read;

/** initialize **/
eofloop = 0;
ndraws  = 0;
a2q2 = 0;

/** discard initial draws **/
pararow = seekr(fhpara,nburn);
statrow = seekr(fhstat,nburn);

do until eofloop;

   /** read block from MH output **/
   parasim = readr( fhpara, nblock );
   statsim = readr( fhstat, nblock );
 
   a2q2sim = zeros(rows(parasim),1);

    i=1;
    do until i > rows(parasim);
        parag = parasim[i,.]';
        postg = statsim[i,1];

        /**  a(para(g),para(*)|Y)  **/
        a2 = minc(1 | exp( post[1] - postg));

        /** log(q(para(g),para(*)|Y)) **/
       q2 = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc^2)
             -0.5*(para - parag)'*SIGPROPINV*(para - parag)/cc^2;

        /**  \sum_{g=1}^M a(para(g),para(*)|Y)*q(para(g),para(*)|Y) **/
        a2q2 = a2q2 + a2*exp(q2);
        
        a2q2sim[i] =  a2*exp(q2);

        i = i+1;
    endo;
    
    ndraws  = ndraws + rows(parasim);       /** number of draws **/

    if _verbose;
        ndraws;; pause(1e-20);
    endif;
    
    writer(fhcjdda2q2,a2q2sim);

   eofloop = eof(fhpara);
endo;

/******************************************************************
* Step 3: evaluate data density
******************************************************************/
pym  = post[1] - ln(a2q2/ndraws) + ln(meanc(a1));


/** print output **/
if _verbose;
    "";"";
     "======================";
    "Model:  ";; $lmodel;
    "Prior:  ";; $lprior;
    "MhRun:  ";; $mhrun;
    "Dataset:";; $dataselstr;
    "======================";
    "";

    "Marginal Data Density";; pym;
     "";
endif;


/** save output **/
wr = writer(fhcjdd,pym);

closeall fhpara, fhstat, fhcjdd, fhcjdda1, fhcjdda2q2;


/** close log file **/
output file = ^outstr off;




/****************************************************
* PROCEDURES: FCN2
* evaluates posterior density given parameter bounds
****************************************************/
proc (2) = fcn2(para);
local lnpY, lnprio, obsmean, obsvar, lnpost, retcode, parabd_ind1, parabd_ind2;

    parabd_ind1 = para .> _bounds[.,1];         /** lower bounds **/
    parabd_ind2 = para .< _bounds[.,2];         /** upper bounds **/
    parabd_ind1 = delif(parabd_ind1,pmask);
    parabd_ind2 = delif(parabd_ind2,pmask);

    if (parabd_ind1 == 1) and (parabd_ind2 == 1);   /** in bounds **/

        {lnpY,retcode,obsmean,obsvar} = evalmod(para,YY,msel);
        lnpy    = real(lnpy);
        lnprio  = priodens( para, pmean, pstdd, pshape);
        lnpost  = lnpy + real(lnprio);

    else;                                                           /** out of bounds **/

        lnpost = -1E6;
        lnpy   = -1E20;

    endif;

retp(lnpost,lnpy);
endp;
