/******************************************************************************
* NKMP_PMPLOT.G
* plots posterior density
*
* upadted:  06/02/2005
******************************************************************************/
new;

/** set seed **/
/** random number generator used in optimization procedure **/
rndseed 1234;

/********************************************************
* House cleaning and set defaults
********************************************************/
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
__output = 1;       /** 1 for verbose of CSMIN**/
__graph  = 1;       /** 1 for graph mode **/
_verbose = 2;       /** 0 for silence, 1 for compact, 2 for detailed**/
_fcmptol = 1E-10;

/** only the following 2 lines are model dependent **/
#include nkmp_mod.src;
#include nkmp_para.src;
#include nkmp_spec.g;

#include pathspec.g;

/********************************************************
*  Parameter Grid
********************************************************/

/* tau */

/* baseline: lowerbound = 1000, para_n = 41 */

para1_min = 0.5;
para1_max = 3;
para1_n = 41;
para1_grid = seqa( para1_min, (para1_max-para1_min)/(para1_n-1), para1_n);

/* psi2 */

para2_min = 0.1;
para2_max = 2.5;
para2_n = 41;
para2_grid = seqa( para2_min, (para2_max-para2_min)/(para2_n-1), para2_n);

lowerbound = -1000;

/********************************************************
*  Import prior specification
********************************************************/
priorfile = prioname $+ ".txt";
load path = ^priopath prior[] = ^priorfile;
prior = reshape(prior,rows(prior)/5,5);

{prior,para_names} = psel(prior,msel);

pshape   = prior[.,1];
pmean    = prior[.,2];
pstdd    = prior[.,3];
pmask    = prior[.,4];
pfix     = prior[.,5];
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

/********************************************************
*  Read local modes
********************************************************/

pmfile = postpath $+ "\\pm1.dat";
open fhpara = ^pmfile for read;
parapm1 = readr(fhpara,1)';
closeall fhpara;

pmfile = postpath $+ "\\pm2.dat";
open fhpara = ^pmfile for read;
parapm2 = readr(fhpara,1)';
closeall fhpara;

/********************************************************
* Import data series
********************************************************/
#include loaddata.g;

nobs  = rows(YY);          /** number of observations **/
ny    = cols(YY);          /** number of series **/

/********************************************************
* Evaluate posterior / likelihood on Grid
********************************************************/

prior1_grid = zeros(para1_n,para2_n);
prior2_grid = zeros(para1_n,para2_n);

like1_grid = zeros(para1_n,para2_n);
like2_grid = zeros(para1_n,para2_n);

post1_grid = zeros(para1_n,para2_n);
post2_grid = zeros(para1_n,para2_n);


para1_ind = 1;

do until para1_ind > para1_n;

   para1 = para1_grid[para1_ind];
   para1 = para1;

   "Step: " para1_ind;
   pause(1E-5);
   
   para2_ind = 1;
   
   do until para2_ind > para2_n;
   

      para2 = para2_grid[para2_ind];
      para2 = para2 - 0.0057;

      modelpara1 = parapm1;
      modelpara1[1] = para1;
      modelpara1[4] = para2;
      
      modelpara2 = parapm2;
      modelpara2[1] = para1;
      modelpara2[4] = para2;
      
      {lnpY,retcode,obserror,obsvar} = evalmod(modelpara1,YY,msel);
      lnprio = priodens(modelpara1, pmean, pstdd, pshape);

      like1_grid[para1_ind,para2_ind] = maxc(lnpY|lowerbound);
      prior1_grid[para1_ind,para2_ind] = lnprio;
      post1_grid[para1_ind,para2_ind] = maxc(lnpY+lnprio|lowerbound);
      
      {lnpY,retcode,obserror,obsvar} = evalmod(modelpara2,YY,msel);
      lnprio = priodens(modelpara2, pmean, pstdd, pshape);
           
      like2_grid[para1_ind,para2_ind] = maxc(lnpY|lowerbound);
      prior2_grid[para1_ind,para2_ind] = lnprio;
      post2_grid[para1_ind,para2_ind] = maxc(lnpY+lnprio|lowerbound);
      
      para2_ind = para2_ind+1;
   endo;
   
   para1_ind = para1_ind+1;
endo;


/********************************************************
* Plot Results
********************************************************/

graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.4;
_pnumht = 0.4;
_pltype = {6 1 2}; 
_pcolor = {1 4};
_plwidth = 3.5;
_plctrl = -1;
_protate = 1;
begwind;
window(3,2,1);


ylabel("Tau");
xlabel("Psi 2");


ytics(0,3,1,1);
xtics(0,2.5,0.5,1);


title("Cut 1 - Prior Contour");

ztics(-10,0,2,1); /* s5t1 */

contour(para2_grid',para1_grid,prior1_grid); 
nextwind;

title("Cut 2 - Prior Contour");

contour(para2_grid',para1_grid,prior2_grid); 
nextwind;

title("Cut 1 - Likelihood Contour");

ztics(-500,50,100,1); /* m11s5t1 */
/* ztics(-300,50,25,1); */ /* m13s5t1 */
/* ztics(-1000,-400,100,1); */ /* m11s5t2 */

contour(para2_grid',para1_grid,like1_grid); 
nextwind;

title("Cut 2 - Likelihood Contour");

contour(para2_grid',para1_grid,like2_grid); 
nextwind;

title("Cut 1 - Posterior Contour");

contour(para2_grid',para1_grid,post1_grid); 
nextwind;

title("Cut 2 - Posterior Contour");

contour(para2_grid',para1_grid,post2_grid); 
endwind;

save path = ^postpath para2_grid, para1_grid, post1_grid, post2_grid;

end;
