/******************************************************************************
* NKMP_MHCHK.G
* generates draws from the prior distribution
* via the random walk Metropolis-Hastings algorithm / sampling from p(Y|theta)
*
* upadted:  06/16/2005 
******************************************************************************/

new;
/** set seed **/
/** rndseed 1234; **/


/********************************************************
* House cleaning and set defaults
********************************************************/
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
_verbose = 1;       /** 1 for verbose **/

#include nkmp_mod.src;
#include nkmp_para.src;
#include nkmp_spec.g;
#include nkmp_sim.src
#include pathspec.g;

/** start log file **/
outstr = resupath $+ "\\mhchk.log";
output file = ^outstr reset;

nobs_burn  = 100;          /** number of observations **/
nobs_total = 180;

/********************************************************
*  Import prior specification
********************************************************/
priorfile = prioname $+ ".txt";
load path = ^priopath prior[] = ^priorfile;
prior = reshape(prior,rows(prior)/5,5);

{prior,para_names} = psel(prior,msel);

pshape   = prior[.,1];
pmean    = prior[.,2];
pstdd    = prior[.,3];
pmask    = prior[.,4];
pfix     = prior[.,5];
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

/** parameter specification: transformation and bounds **/
_trspec      = prtr(msel);
_trspec[.,1] = _trspec[.,1].*pmaskinv;
_bounds      = _trspec[.,2 3];


/********************************************************
* Load Posterior Mode Estimates
********************************************************/
lpara = postpath $+ "\\pm";
lretc = postpath $+ "\\pmret";

/** check posterior mode results **/
if not(exist(lpara $+ ".dat"));
    "";
    "Please run posterior maximization program first.";
    "";
    end;
endif;

/** open **/
open fhpara = ^lpara for read;
para_pm = readr(fhpara,1);
closeall fhpara;

para_pm  = para_pm';          /** paraest was saved as row vector in _pm.g **/
npara = rows(para_pm);

open fhretc = ^lretc for read;
_csretcode = readr(fhretc,1);
closeall fhretc;

/********************************************************
* Load multiplication matrix for parameters
********************************************************/
lmult = postpath $+ "\\mult";       /** sigmult **/
lhess = postpath $+ "\\hess";     /** HHM **/


/** check Hessian results **/
if not(exist(lmult $+ ".dat"));
    "";
    "Please run Hessian finding program first.";
    "";
    end;
endif;

/** open **/
open fhmult = ^lmult for read;
SIGSCALE    = readr( fhmult,npara );
closeall fhmult;

open fhhess = ^lhess for read;
SIGPROPINV  = readr( fhhess,npara );
closeall fhhess;

/** rank of Hessian (theoretical) **/
SIGPROPdim = sumc(pmaskinv);

/** inverse by Singular Value Decomposition (SVD) **/
{u , s, v} = svd1(SIGPROPINV);
SIGPROPlndet = 0;

/** log determinant for nonsingular part **/
i = 1;
do until i > npara;
   if i <= SIGPROPdim;
      s[i,i]    = 1/s[i,i];
      SIGPROPlndet = SIGPROPlndet + ln(s[i,i]);
   endif;
   i = i+1;
endo;


/********************************************************
* Configure the Metropolis algorithm
********************************************************/

mhrun   = "1";

nblocks = 20;
nsim    = 1000;

cc0     = 1;
cc      = 0.15;  

/* 
** use cc = 0.15 for usm11t0p1
*/

appendmode = 0;
paraold = pmean;

opath   = parapath $+ "\\mhrun" $+ mhrun;
chkdir  = chk_dir(opath);

/** print specification **/
if _verbose;

    "";
    "===================================";
    "Checking the MCMC procecure";
    "Model            " $lmodel;
    "Data set         " $dataselstr;
    "Prior            " $lprior;
    "Subsample        " $subTstr;
    "MHrun Index      " $mhrun;
    "Scaling of Hess  " cc;
    "";
    "===================================";
    "";
    
endif;      /** _verbose **/



/********************************************************
* Initialize Output files
********************************************************/

opara = opath $+ "\\pachk";

/** check if append is possible **/
if not(exist(opara $+ ".dat"));
    appendmode=0;
endif;

if appendmode NE 1;

  /*******************************
  ** Create new output files
  *******************************/

  "-----------------------------------------------------------";
  "Creating new output files";
  "Press Key to Continue";
  "-----------------------------------------------------------";
  k = keyw;
  "";
  
  create fhpara = ^opara with ^para_names, npara, 4;

else;

  /*******************************
  ** Open existing output files
  *******************************/

  "-----------------------------------------------------------";
  "Opening Existing Output Files";
  "Press Key to Continue";
  "-----------------------------------------------------------";

  open fhpara = ^opara for update;
  
  rowind = seekr(fhpara,rowsf(fhpara));

  paraold    = readr(fhpara,1)';

  "Number of draws in File" rowind;
  "";

  rowind = seekr(fhpara,0);

  cc0 = 0;

  "Parameter   | Last Draw ";
  outmat = para_names'~paraold;
  let mask[1,2] = 0 1;
  let fmt[2,3] =
               "-*.*s " 10 6
               "*.*lf " 10 4;
  d = printfm(outmat,mask,fmt);

  "";
  "Press Key to Continue";
  "-----------------------------------------------------------";
  k = keyw;
  
endif;

/********************************************************
* Generate draws from joint of parameters and data 
********************************************************/


/** block loop **/
iblock = 1;
do until iblock > nblocks;
    
   if _verbose;
   "";
   "----------------------------------------";
   "Block" iblock "of" nblocks;
   pause(1e-20);
   endif;

   /** set start time **/
   tstart = date;

   /** storages **/
   parasim = zeros(nsim,npara);
   rej = zeros(nsim,1);
   
   j=1;

   /** with-in block loop **/
   do until j > nsim;

      YY = dsgesim(paraold,msel,nobs_burn,nobs_total);
      {postold, likeold} = fcn2(paraold);

      /** propose **/
      paranew = paraold + cc*(sigscale*rndn(npara,1));
      paranew = paranew.*pmaskinv + pfix.*pmask;

      /** evaluate at the candidate **/
      {postnew, likenew} = fcn2(paranew);

      /** accept/reject rule  **/
      r = minc(1 | exp( postnew - postold));

      if rndu(1,1) < r;

          /** accept proposed jump **/
          parasim[j,.] = paranew';

          /** updates **/
          paraold = paranew;

      else;

          /** reject proposed jump **/
          parasim[j,.] = paraold';
          rej[j] = 1;

      endif;

        if _verbose;
            if fmod(j,100)==0;".";; pause(1e-20); endif;
        endif;

      j = j+1;
    endo;               /** with-in block  **/

   if _verbose;
       " ";
       "Rejection: perct " meanc(rej);
       "Parameters       "; meanc(parasim)';
       "";
       pause(1e-20);
   endif;  /** verbose **/

   /** set finish time **/
   tfin = date;

   if _verbose;
       "Elapsed Time"; etstr(ethsec(tstart,tfin));
       "";
   endif;  /** verbose **/

   /** save output **/
   wr = writer(fhpara,parasim);

   iblock = iblock+1;

endo;           /** block **/

closeall fhpara;

/** close log file **/
output file = ^outstr off;


/****************************************************
* PROCEDURES: FCN2
* evaluates posterior density given parameter bounds
****************************************************/
proc (2) = fcn2(para);
local lnpY, lnprio, obsmean, obsvar, lnpost, retcode, parabd_ind1, parabd_ind2;

    parabd_ind1 = para .> _bounds[.,1];         /** lower bounds **/
    parabd_ind2 = para .< _bounds[.,2];         /** upper bounds **/
    parabd_ind1 = delif(parabd_ind1,pmask);
    parabd_ind2 = delif(parabd_ind2,pmask);

    if (parabd_ind1 == 1) and (parabd_ind2 == 1);   /** in bounds **/

        {lnpY,retcode,obsmean,obsvar} = evalmod(para,YY,msel);
        lnpy    = real(lnpy);
        lnprio  = priodens( para, pmean, pstdd, pshape);
        lnpost  = lnpy + real(lnprio);

    else;                                                           /** out of bounds **/

        lnpost = -1E6;
        lnpy   = -1E20;

    endif;

retp(lnpost,lnpy);
endp;
