/******************************************************************************
* NKMP_IS.G
* generates draws from the posterior distribution
* via the importance sampler
*
* upadted:  07/08/2005
******************************************************************************/

new;
/** set seed **/
/** rndseed 1234; **/


/********************************************************
* House cleaning and set defaults
********************************************************/
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
_verbose = 1;       /** 1 for verbose **/

pmsel   = "";
isrun   = "13";

#include nkmp_mod.src;
#include nkmp_para.src;
#include nkmp_spec.g;

#include pathspec.g;

/** log file **/
outstr = resupath $+ "\\is.log";

#include head.g;

/********************************************************
* Configure the importance sampler
********************************************************/

iter    = 1;         /** 1  for starting from posterior mode,
                         2  for successive iteration   **/

varscale = 1.2;        /* scale variance */
cc      = 3;         /** df for t-dist**/

nsim    = 10000;      /** number of draws in each block **/
nblocks = 500;      /** number of blocks **/

/** output path **/
opath   = parapath $+ "\\isrun" $+ isrun;
chkdir  = chk_dir(opath);

/** ckeck the previous results **/
pasfile = opath $+ "\\pas*";

if not exist(pasfile);

   if _verbose;
      " Cannot find the results from previous run.";
      " Press Key to start from the posterior mode.";
      k = keyw;
   endif;

   iter = 1;                        /** set iteration as 1 **/
   output file = ^outstr reset;     /** reset log file **/

endif;


/********************************************************
* Initialize the importance sampler:
* moments for importance density
********************************************************/
if iter > 1;

   /** resume log file **/
   output file = ^outstr on;        /** resume log file **/

   /** check the last run **/
   paslist = filesa(pasfile);
   iter = stof(strsect(paslist[rows(paslist),.],4,2))+1;

   /** use previous iteration results for initialization **/
   lpas = opath $+ "\\pas" $+ ftos(iter-1,"%0*.*lf",2,0);
   lmax = opath $+ "\\nwmax" $+ ftos(iter-1,"%0*.*lf",2,0);

   /** check maximum weight from previous run **/
   open fhmax = ^lmax for read;
   nw_max = readr(fhmax,1);
   closeall fhmax;


   "";
   "Maximum weight from the previous run is " 100*nw_max " percent";
   "Press ENTER to continue the next run.";
   k = keyw;

   if not(k==13);
      "";
      "User terminates the importance sampler.";
      "";
      end;
   endif;

   open fhpas = ^lpas for read;
      pastat = readr(fhpas,npara+4);
   closeall fhpas;

   meanit     = pastat[1,.]';                         /** mean **/
   varit      = pastat[3:npara+2,.]*varscale;                  /** variance **/
   cholvarit  = sqrt((cc-2)/cc)*chol(varit);          /** Choleski decomp **/
   varinvit   = cc/(cc-2)*inv(varit);                 /** inverse **/
   varlndetit = npara*( ln((cc-2)/cc) )+ln(det(varit));   /** log determinant **/

elseif iter==1;

   /** use posterior mode for initialization **/
   meanit     = para;                                 /** initial mean **/
   cholvarit  = sqrt((cc-2)/cc)*SIGSCALE*sqrt(varscale);             /** Choleski decomp **/
   varinvit   = cc/(cc-2)*SIGPROPINV/varscale;         /** inverse **/
   varlndetit = npara*( ln((cc-2)/cc) + ln(varscale) )+SIGPROPlndet;     /** log determinant **/

endif;


/** print specification **/
if _verbose;
   "";
   "Posterior Mode Info:";
   "===================================";
   "Model            " $lmodel;
   "Data set         " $dataselstr;
   "Prior            " $lprior;
   "Subsample        " $subTstr;
   "ISrun Index      " $isrun;
   "df of t-dist     " cc;
   "";
   "===================================";
   "";

   "";
   "Parameter   | Estimate ";
   outmat = para_names'~para;
   let mask[1,2] = 0 1;
   let fmt[2,3]  =
                   "-*.*s " 10 6
                   "*.*lf " 10 4;
   d = printfm(outmat,mask,fmt);
   {lnpost0, lnpy0} = fcn3(para);

   "";
   "Prior*Likelihood at Mode:" lnpost0;
   "Likelihood at Mode      :" lnpy0;
   "";
   "Press Key to Continue";
   k = keyw;
   "";

endif;      /** _verbose **/


/** output file **/
ostat = opath $+ "\\st" $+ ftos(iter,"%0*.*lf",2,0);
opara = opath $+ "\\pa" $+ ftos(iter,"%0*.*lf",2,0);

statname = { POST LIKE IMPDEN WEIGHT };
create fhstat = ^ostat with ^statname, 4, 4;
create fhpara = ^opara with ^para_names, npara, 4;





/********************************************************
* Block loop
********************************************************/
/** set time **/
tstart0 = date;

iblock=1;
do until iblock > nblocks;

   /** print process **/
   if _verbose;
     "";
     "----------------------------------------";
     "Block" iblock "of" nblocks;
     pause(1e-20);
   endif;

   /** set start time **/
   tstart1 = date;

   /** storages **/
   likesim = zeros(nsim,1);
   postsim = zeros(nsim,1);
   impdsim = zeros(nsim,1);
   weightsim = zeros(nsim,1);


   /********************************************************
   * Importance Sampler
   ********************************************************/
   /** draws from Gamma(cc/2,cc/2) **/
   tauvec = cc/2*rndgam(nsim,1,cc/2);

   /** draws from conditional normal **/
   parasim = (meanit + cholvarit*rndn(npara,nsim)./sqrt(tauvec'))';

   /** evaluate weights **/
   j=1;
   do until j > nsim;            /** with-in block loop **/

      paranew = parasim[j,.]';
      paranew = paranew.*pmaskinv + pfix.*pmask;

      /** evaluate at the draws **/
      {postnew, likenew} = fcn3(paranew);

      impdens = lnfact((cc+npara)/2) - lnfact(npara/2) - 0.5*varlndetit - npara/2*ln(cc*pi)
                - (cc+npara)/2*ln(1+(paranew-meanit)'*varinvit*(paranew-meanit)/cc);

      likesim[j,1] = likenew;
      postsim[j,1] = postnew;

      impdsim[j,1] = impdens*likenew/likenew;         /** control out-of-bounds as missing **/
      parasim[j,.] = paranew'*likenew/likenew;

      weightsim[j,1] = postnew-impdens;

      if _verbose;
         if fmod(j,100)==0;".";; pause(1e-20); endif;
      endif;

      j = j+1;
   endo;               /** with-in block  **/

   /** dismiss out-of-bounds draws **/
   parasim = packr(parasim);
   likesim = packr(likesim);
   postsim = packr(postsim);
   impdsim = packr(impdsim);
   weightsim = packr(weightsim);

   /** set finish time **/
   tfin = date;

   /** print process **/
   if _verbose;
      " ";
      "In-bound draws    " rows(likesim);
      "Max Weight(perct) " 100/sumc(exp(weightsim-maxc(weightsim)));
      "Likelihood min/max" minc(likesim) maxc(likesim);
      "Posterior min/max " minc(postsim) maxc(postsim);
      "Parameters        "; meanc(parasim)';
      "";
      "Elapsed Time (Block)     " etstr(ethsec(tstart1,tfin));
      "Elapsed Time (Total)     " etstr(ethsec(tstart0,tfin));
      "";
      pause(1e-20);
   endif;  /** verbose **/

   /** save output **/
   wr = writer(fhstat,postsim~likesim~impdsim~weightsim);
   wr = writer(fhpara,parasim);

   iblock = iblock+1;

endo;           /** block **/

closeall fhstat, fhpara;



block_size  = 10000;
/*************************************************************************
*     Normalize weights
*************************************************************************/
open fhstat = ^ostat for read;

drawblock = readr( fhstat, 1);
drawcol   = drawblock[1,4];

eofloop   = 0;
do until eofloop;
   drawblock = readr( fhstat, block_size );
   drawcol   = drawcol | drawblock[.,4];
   eofloop   = eof(fhstat);
endo;

closeall fhstat;

drawcol = drawcol - maxc(drawcol);
nwgt = exp(drawcol)/sumc(exp(drawcol));

/** save normalized weights **/
onwgt = opath $+ "\\nwgt" $+ ftos(iter,"%0*.*lf",2,0);
create fhnwgt = ^onwgt with NWGT, 1, 8;
wr = writer(fhnwgt,nwgt);
closeall fhnwgt;





/*************************************************************************
*     Resample to equally-weighted samples
*************************************************************************/

if _verbose;
	"";
	"";
	"----------------------------------------------------------";
	"Resampling to eqaully-weighted samples...";

	"";
endif;


/********************************************************
*     Construct resampling index
********************************************************/
open fhnwgt = ^onwgt for read;

drawcol = readr(fhnwgt,1);

eofloop = 0;
ndraws  = 1;
do until eofloop;

	drawblock = readr( fhnwgt, block_size );
   drawcol   = drawcol | drawblock;
   ndraws    = ndraws + rows(drawblock);
   eofloop   = eof(fhnwgt);

endo;

newidx = resmpl(drawcol);

closeall fhnwgt;


/********************************************************
*     Assign according to newidx by parameter 
********************************************************/

open fhdraw = ^opara for read;

j = 1;
do until j > npara;

   if _verbose;
      "Column" j;
   endif;

   /* Read only the j'th column */
	drawblock = seekr( fhdraw, 1 );
	drawblock = readr( fhdraw, 1 );

   drawcol = drawblock[1,j];

   eofloop = 0;
   ndraws  = 1;
   do until eofloop;

      drawblock = readr( fhdraw, block_size );
      drawcol   = drawcol | drawblock[.,j];
      ndraws    = ndraws + rows(drawblock);
      eofloop   = eof(fhdraw);

   endo;

	drawcol = drawcol[newidx];

	/** save new samples **/
	oisrsx = opath $+ "\\isrs" $+ ftos(iter,"%0*.*lf",2,0) $+ "_" $+ ftos(j,"%0*.*lf",2,0);
	create fhisrsx = ^oisrsx with ISRSX, 1, 8;
	wr = writer(fhisrsx,drawcol);
	cf = close(fhisrsx);

	if _verbose;
		"ndraws  " ndraws;
	endif;

	j = j+1;

endo;

closeall fhdraw;


/********************************************************
*     Collect equally-weighted draws into one file
********************************************************/
oisrs = opath $+ "\\isrs" $+ ftos(iter,"%0*.*lf",2,0);
create fhisrs = ^oisrs with ISRS, npara, 8;

drawrow = 1;
eofloop = 0;
do until eofloop;

	drawcol = {};

	j=1;
	do until j>npara;
		oisrsx = opath $+ "\\isrs" $+ ftos(iter,"%0*.*lf",2,0) $+ "_" $+ ftos(j,"%0*.*lf",2,0);

		open fhisrsx = ^oisrsx for read;
		drawblock = seekr(fhisrsx,drawrow);
		drawblock = readr(fhisrsx,block_size);
		eofloop   = eof(fhisrsx);
		cf = close(fhisrsx);

		drawcol = drawcol ~ drawblock;

		j=j+1;
	endo;

	wr = writer(fhisrs,drawcol);
	drawrow = drawrow + rows(drawcol);

endo;

closeall fhisrs;






/*************************************************************************
*     Compute posterior mean and std and CI
*************************************************************************/
hpdprob     = 0.90;


/********************************************************
*     Load Parameter Draws from IS Output
********************************************************/
open fhdraw = ^opara for read;
open fhnwgt = ^onwgt for read;


/************************************************
*  Part 1: Compute the mean of x(i) and x^2(i)
************************************************/
drawmean   = zeros(1,npara);
drawsqmean = zeros(1,npara);
drawcross  = zeros(npara,npara);

eofloop = 0;
ndraws  = 0;
weightsum = 0;

if _verbose;
   "";
   "===================================";
   "Part 1: Calculate mean and variance";
   "===================================";
   "";
endif;

do until eofloop;

   drawblock1 = readr( fhdraw, block_size );
   drawblock2 = readr( fhnwgt, block_size );

   ndraws     = ndraws     + rows(drawblock1);
   drawmean   = drawmean   + sumc(drawblock1.*drawblock2)';
   drawsqmean = drawsqmean + sumc((drawblock1^2).*drawblock2)';
   drawcross  = drawcross  + drawblock1'*(drawblock1.*drawblock2);

   if _verbose;
      "  Draws " ndraws;
   endif;

   eofloop = eof(fhdraw);

endo;

drawmean   = drawmean;
drawsqmean = drawsqmean;
drawstdd   = sqrt(drawsqmean - (drawmean)^2);
drawcov    = drawcross - drawmean'*drawmean;



/************************************************
*  Part 2: HPD Interval
************************************************/
drawci      = zeros(2,npara);

if _verbose;
   "";
   "===================================";
   "Part 2: HPD CI                     ";
   "===================================";
   "";
endif;

j = 1;
do until j > npara;

   if _verbose;
      "Column" j;
   endif;

   /* Read only the j'th column */
   drawrow1   = seekr( fhdraw, 1);
   drawrow2   = seekr( fhnwgt, 1);

   drawblock1 = readr( fhdraw, 1);
   drawblock2 = readr( fhnwgt, 1);

   drawcol = drawblock1[1,j]~drawblock2[1,1];

   eofloop = 0;
   ndraws  = 1;

   do until eofloop;

      drawblock1 = readr( fhdraw, block_size );
      drawblock2 = readr( fhnwgt, block_size );

      drawcol   = drawcol | (drawblock1[.,j]~drawblock2);
      ndraws    = ndraws + rows(drawblock1);
      eofloop   = eof(fhdraw);

      if _verbose;
         "ndraws  " ndraws;
      endif;

   endo;

   drawci[.,j] = hpdintw(drawcol,hpdprob);
   j = j+1;

endo;


/************************************************
*  Part 3: max weight
************************************************/
nw_max = maxc(drawcol[.,2]);




/************************************************
*  Report Posterior Mean and stderror
************************************************/
"";
"";
"=============================================";
"MODEL       " lmodel;
"PRIOR       " lprior;
"ISRun       " isrun;
"Subsample   " subTstr;
"Coverage    " hpdprob;
"Dataset     " dataselstr;
"=============================================";
" ";
"Max weight (percent) " 100*nw_max;
"";
"";

"Parameter | Mean  |  StdD | CI(LOW) | CI(HIGH)";
outmat = para_names'~ drawmean' ~ drawstdd' ~ drawci' ;
let mask[1,5] = 0 1 1 1 1;
let fmt[5,3] =
   "-*.*s " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4;

d = printfm(outmat,mask,fmt);
"";
"";


/************************************************
** save output
************************************************/
odraws = opath $+ "\\pas" $+ ftos(iter,"%0*.*lf",2,0);
create fhdraws = ^odraws with DRAWSUM, npara, 8;
wr = writer(fhdraws, real(drawmean | drawstdd | drawcov | drawci) );
closeall fhdraws;

onwmax = opath $+ "\\nwmax" $+ ftos(iter,"%0*.*lf",2,0);
create fhnwmax = ^onwmax with NWMAX, 1, 8;
wr = writer(fhnwmax, nw_max );
closeall fhnwmax;



/** close log file **/
output file = ^outstr off;



end;


/****************************************************
* PROCEDURES: FCN3
* evaluates posterior density given parameter bounds
****************************************************/
proc (2) = fcn3(para);
local lnpY, lnprio, obsmean, obsvar, lnpost, retcode, parabd_ind1, parabd_ind2;

    parabd_ind1 = para .> _bounds[.,1];         /** lower bounds **/
    parabd_ind2 = para .< _bounds[.,2];         /** upper bounds **/
    parabd_ind1 = delif(parabd_ind1,pmask);
    parabd_ind2 = delif(parabd_ind2,pmask);

    if (parabd_ind1 == 1) and (parabd_ind2 == 1);   /** in bounds **/

        {lnpY,retcode,obsmean,obsvar} = evalmod(para,YY,msel);
        lnpy    = real(lnpy);
        lnprio  = priodens( para, pmean, pstdd, pshape);
        lnpost  = lnpy + real(lnprio);

        if lnpy < -1E8+1;
          lnpost = miss(0,0);
          lnpy   = miss(0,0);
        endif;

    else;                                                           /** out of bounds **/

        lnpost = miss(0,0);
        lnpy   = miss(0,0);

    endif;

retp(lnpost,lnpy);
endp;


/****************************************************
* PROCEDURES: HPDINTW
* calculates HPD interval for samples with weights
****************************************************/
proc(1) = hpdintw(draws,percent);
local     w, newidx, hpdband;

/** weights are on the last column **/
w = draws[.,cols(draws)];

/** resampling: equally-weighted draws **/
newidx = resmpl(w);
draws  = draws[newidx,.];

/** calculate hpd **/
hpdband = hpdint(draws[.,1:(cols(draws)-1)],percent);

retp(hpdband);
endp;


/****************************************************
* PROCEDURES: HPDINT
* calculates HPD interval for equally-weighted samples
****************************************************/
proc(1) = hpdint(draws,percent);
local     ndraws, drawdim, hpdband, nwidth, i, drawcoli,
          bup, minwidth, newwidth, j;

drawdim   = cols(draws);
ndraws    = rows(draws);
hpdband   = zeros(2,drawdim);
nwidth    = int(percent*ndraws);

i = 1;
do until i > drawdim;
   drawcoli = draws[.,i];
   /* sort response for period i, element 1 is max
   */
   drawcoli = rev(sortc(drawcoli,1));
   bup   = 1;
   minwidth  = drawcoli[1] - drawcoli[nwidth];
   j = 2;
   do until j > (ndraws-nwidth+1);
      newwidth = drawcoli[j] - drawcoli[j+nwidth-1];
      if newwidth < minwidth;
         bup = j;
         minwidth = newwidth;
      endif;
      j = j+1;
   endo;
   hpdband[2,i] = drawcoli[bup];
   hpdband[1,i] = drawcoli[bup+nwidth-1];
   i = i+1;
endo;
retp(hpdband);
endp;


/****************************************************
* PROCEDURES: RESMPL
% makes resampled index
% resamples with weights, with replacement
% based on Kitagawa's (1996) deterministic resampling algorithm
****************************************************/
proc(1) = resmpl(w);
local     nw, idx, cdf, u, j;


/** initialize **/
nw  = rows(w);
idx = zeros(nw,1);

/** construct CDF **/
cdf = cumsumc(w);

/** draw a starting point **/
u = (rndu(1,1)-1+seqa(1,1,nw))/nw;

/** start at the bottom of the CDF **/
i=1;j=1;
do until i>nw;

   /** move along the CDF **/
   do while (u[i]>cdf[j]);
      j=j+1;
   endo;

   /** assign index **/
   idx[i] = j;

   i=i+1;
endo;

retp(idx);
endp;



